"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
const shared_1 = require("@vue/shared");
const template_1 = require("./template");
function generate(node) {
    let text = '';
    const tags = new Set();
    for (const child of node.children) {
        visitNode(child);
    }
    return {
        text,
        tags,
    };
    function visitNode(node) {
        var _a, _b, _c;
        if (node.type === 1 /* ELEMENT */) {
            const patchForNode = (0, template_1.getPatchForSlotNode)(node);
            if (patchForNode) {
                visitNode(patchForNode);
                return;
            }
            tags.add(node.tag);
            text += `{\n`;
            for (const prop of node.props) {
                if (prop.type === 7 /* DIRECTIVE */) {
                    // arg
                    if (((_a = prop.arg) === null || _a === void 0 ? void 0 : _a.type) === 4 /* SIMPLE_EXPRESSION */ && !prop.arg.isStatic) {
                        text += `// @ts-ignore\n`;
                        text += `(${wrapTsIgnores(prop.arg.loc.source)});\n`;
                    }
                    // exp
                    if (((_b = prop.exp) === null || _b === void 0 ? void 0 : _b.type) === 4 /* SIMPLE_EXPRESSION */) {
                        if (prop.name === 'slot') {
                            text += `// @ts-ignore\n`;
                            text += `let ${wrapTsIgnores(prop.exp.content)} = {} as any;\n`;
                        }
                        else if (prop.name === 'on') {
                            text += `// @ts-ignore\n`;
                            text += `() => { ${wrapTsIgnores(prop.exp.content)} };\n`;
                        }
                        else {
                            text += `// @ts-ignore\n`;
                            text += `(${wrapTsIgnores(prop.exp.content)});\n`;
                        }
                    }
                    // name
                    if (prop.name !== 'slot'
                        && prop.name !== 'on'
                        && prop.name !== 'model'
                        && prop.name !== 'bind') {
                        text += `// @ts-ignore\n`;
                        text += `(${(0, shared_1.camelize)('v-' + prop.name)});\n`;
                    }
                }
                else if (prop.type === 6 /* ATTRIBUTE */
                    && prop.name === 'ref'
                    && prop.value) {
                    text += `// @ts-ignore\n`;
                    text += `(${prop.value.content});\n`;
                }
            }
            for (const childNode of node.children) {
                visitNode(childNode);
            }
            text += '}\n';
        }
        else if (node.type === 12 /* TEXT_CALL */) {
            // {{ var }}
            visitNode(node.content);
        }
        else if (node.type === 8 /* COMPOUND_EXPRESSION */) {
            // {{ ... }} {{ ... }}
            for (const childNode of node.children) {
                if (typeof childNode === 'object') {
                    visitNode(childNode);
                }
            }
        }
        else if (node.type === 5 /* INTERPOLATION */) {
            // {{ ... }}
            const context = node.loc.source.substring(2, node.loc.source.length - 2);
            text += `// @ts-ignore\n`;
            text += `{ ${context} };\n`;
        }
        else if (node.type === 9 /* IF */) {
            // v-if / v-else-if / v-else
            for (let i = 0; i < node.branches.length; i++) {
                const branch = node.branches[i];
                text += `// @ts-ignore\n`;
                if (i === 0)
                    text += 'if';
                else if (branch.condition)
                    text += 'else if';
                else
                    text += 'else';
                if (((_c = branch.condition) === null || _c === void 0 ? void 0 : _c.type) === 4 /* SIMPLE_EXPRESSION */) {
                    text += ` (${wrapTsIgnores(branch.condition.content)})`;
                }
                text += ` {\n`;
                for (const childNode of branch.children) {
                    visitNode(childNode);
                }
                text += '}\n';
            }
        }
        else if (node.type === 11 /* FOR */) {
            // v-for
            const source = node.parseResult.source;
            const value = node.parseResult.value;
            const key = node.parseResult.key;
            const index = node.parseResult.index;
            if (source.type === 4 /* SIMPLE_EXPRESSION */
                && (value === null || value === void 0 ? void 0 : value.type) === 4 /* SIMPLE_EXPRESSION */) {
                text += `// @ts-ignore\n`;
                text += `for (let ${wrapTsIgnores(value.content)} of ${wrapTsIgnores(source.content, value.content.indexOf('\n') >= 0)}) {\n`;
                if ((key === null || key === void 0 ? void 0 : key.type) === 4 /* SIMPLE_EXPRESSION */) {
                    text += `// @ts-ignore\n`;
                    text += `let ${wrapTsIgnores(key.content)} = {} as any;`;
                }
                if ((index === null || index === void 0 ? void 0 : index.type) === 4 /* SIMPLE_EXPRESSION */) {
                    text += `// @ts-ignore\n`;
                    text += `let ${wrapTsIgnores(index.content)} = {} as any;`;
                }
                for (const childNode of node.children) {
                    visitNode(childNode);
                }
                text += '}\n';
            }
        }
    }
    ;
}
exports.generate = generate;
function wrapTsIgnores(content, dontIgnoreFirstLine = false) {
    if (content.indexOf('\n') === -1)
        return content;
    return content
        .split('\n')
        .map((line, i) => !dontIgnoreFirstLine && i === 0 ? line : `// @ts-ignore\n${line}`)
        .join('\n');
}
//# sourceMappingURL=template_scriptSetup.js.map