"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
const shared_1 = require("@vue/shared");
function register({ sourceFiles }) {
    return (uri) => {
        const sourceFile = sourceFiles.get(uri);
        if (!sourceFile)
            return {
                tag: 'unsure',
                attr: 'unsure',
            };
        return {
            tag: getTagNameCase(sourceFile),
            attr: getAttrNameCase(sourceFile),
        };
        function getAttrNameCase(sourceFile) {
            var _a;
            const attrNames = (_a = sourceFile.getTemplateAttrNames()) !== null && _a !== void 0 ? _a : new Set();
            let hasCamelCase = false;
            let hasKebabCase = false;
            for (const tagName of attrNames) {
                // attrName
                if (tagName !== (0, shared_1.hyphenate)(tagName)) {
                    hasCamelCase = true;
                    break;
                }
            }
            for (const tagName of attrNames) {
                // attr-name
                if (tagName.indexOf('-') >= 0) {
                    hasKebabCase = true;
                    break;
                }
            }
            if (hasCamelCase && hasKebabCase) {
                return 'kebabCase';
            }
            if (hasCamelCase) {
                return 'camelCase';
            }
            if (hasKebabCase) {
                return 'kebabCase';
            }
            return 'unsure';
        }
        function getTagNameCase(sourceFile) {
            var _a;
            const components = sourceFile.getTemplateScriptData().components;
            const tagNames = new Set(Object.keys((_a = sourceFile.getTemplateTagNames()) !== null && _a !== void 0 ? _a : {}));
            let anyComponentUsed = false;
            let hasPascalCase = false;
            let hasKebabCase = false;
            for (const component of components) {
                if (tagNames.has(component) || tagNames.has((0, shared_1.hyphenate)(component))) {
                    anyComponentUsed = true;
                    break;
                }
            }
            if (!anyComponentUsed) {
                return 'unsure'; // not sure component style, because do not have any componnent using in <template> for check
            }
            for (const tagName of tagNames) {
                // TagName
                if (tagName !== (0, shared_1.hyphenate)(tagName)) {
                    hasPascalCase = true;
                    break;
                }
            }
            for (const component of components) {
                // Tagname -> tagname
                // TagName -> tag-name
                if (component !== (0, shared_1.hyphenate)(component) && tagNames.has((0, shared_1.hyphenate)(component))) {
                    hasKebabCase = true;
                    break;
                }
            }
            if (hasPascalCase && hasKebabCase) {
                return 'both';
            }
            if (hasPascalCase) {
                return 'pascalCase';
            }
            if (hasKebabCase) {
                return 'kebabCase';
            }
            return 'unsure';
        }
    };
}
exports.register = register;
//# sourceMappingURL=tagNameCase.js.map