import type * as vscode from 'vscode-languageserver';
import type { TextDocument } from 'vscode-languageserver-textdocument';
import type { HTMLDocument } from 'vscode-html-languageservice';
import type { Stylesheet } from 'vscode-css-languageservice';
import type { PugDocument } from 'vscode-pug-languageservice';
import type { JSONDocument } from 'vscode-json-languageservice';
import * as SourceMaps from '@volar/source-map';
export interface TsMappingData {
    vueTag: 'sfc' | 'template' | 'script' | 'scriptSetup' | 'scriptSrc' | 'style';
    vueTagIndex?: number;
    beforeRename?: (newName: string) => string;
    doRename?: (oldName: string, newName: string) => string;
    capabilities: {
        basic?: boolean;
        extraHoverInfo?: boolean;
        references?: boolean;
        definitions?: boolean;
        diagnostic?: boolean;
        formatting?: boolean;
        rename?: boolean | {
            in: boolean;
            out: boolean;
        };
        completion?: boolean;
        semanticTokens?: boolean;
        foldingRanges?: boolean;
        referencesCodeLens?: boolean;
        displayWithLink?: boolean;
    };
}
export interface TeleportSideData {
    editRenameText?: (newName: string) => string;
    capabilities: {
        references?: boolean;
        definitions?: boolean;
        rename?: boolean;
    };
}
export interface TeleportMappingData {
    isAdditionalReference?: boolean;
    toSource: TeleportSideData;
    toTarget: TeleportSideData;
}
export declare class TsSourceMap extends SourceMaps.SourceMap<TsMappingData> {
    sourceDocument: TextDocument;
    mappedDocument: TextDocument;
    lsType: 'template' | 'script';
    isInterpolation: boolean;
    capabilities: {
        foldingRanges: boolean;
        formatting: boolean;
        documentSymbol: boolean;
        codeActions: boolean;
    };
    constructor(sourceDocument: TextDocument, mappedDocument: TextDocument, lsType: 'template' | 'script', isInterpolation: boolean, capabilities: {
        foldingRanges: boolean;
        formatting: boolean;
        documentSymbol: boolean;
        codeActions: boolean;
    }, mappings?: SourceMaps.Mapping<TsMappingData>[]);
}
export declare class CssSourceMap extends SourceMaps.SourceMap<undefined> {
    sourceDocument: TextDocument;
    mappedDocument: TextDocument;
    stylesheet: Stylesheet | undefined;
    module: string | undefined;
    scoped: boolean;
    links: {
        textDocument: TextDocument;
        stylesheet: Stylesheet;
    }[];
    capabilities: {
        foldingRanges: boolean;
        formatting: boolean;
    };
    constructor(sourceDocument: TextDocument, mappedDocument: TextDocument, stylesheet: Stylesheet | undefined, module: string | undefined, scoped: boolean, links: {
        textDocument: TextDocument;
        stylesheet: Stylesheet;
    }[], capabilities: {
        foldingRanges: boolean;
        formatting: boolean;
    }, mappings?: SourceMaps.Mapping<undefined>[]);
}
export declare class JsonSourceMap extends SourceMaps.SourceMap<undefined> {
    sourceDocument: TextDocument;
    mappedDocument: TextDocument;
    jsonDocument: JSONDocument;
    constructor(sourceDocument: TextDocument, mappedDocument: TextDocument, jsonDocument: JSONDocument, mappings?: SourceMaps.Mapping<undefined>[]);
}
export declare class HtmlSourceMap extends SourceMaps.SourceMap<undefined> {
    sourceDocument: TextDocument;
    mappedDocument: TextDocument;
    htmlDocument: HTMLDocument;
    language: 'html';
    constructor(sourceDocument: TextDocument, mappedDocument: TextDocument, htmlDocument: HTMLDocument, language?: 'html', mappings?: SourceMaps.Mapping<undefined>[]);
}
export declare class PugSourceMap extends SourceMaps.SourceMap<undefined> {
    sourceDocument: TextDocument;
    mappedDocument: TextDocument;
    pugDocument: PugDocument;
    language: 'pug';
    constructor(sourceDocument: TextDocument, mappedDocument: TextDocument, pugDocument: PugDocument, language?: 'pug');
}
export declare class TeleportSourceMap extends SourceMaps.SourceMap<TeleportMappingData> {
    document: TextDocument;
    allowCrossFile: boolean;
    constructor(document: TextDocument, allowCrossFile: boolean);
    findTeleports(start: vscode.Position, end?: vscode.Position): {
        data: TeleportMappingData;
        sideData: TeleportSideData;
        start: vscode.Position;
        end: vscode.Position;
    }[];
    findTeleports2(start: number, end?: number): {
        data: TeleportMappingData;
        sideData: TeleportSideData;
        start: number;
        end: number;
    }[];
}
export * from '@volar/source-map';
